@@ -225,7 +225,7 @@ arm_init(int when)
 		/*
 		 * Kernel text starts 16k after PAGE_OFFSET.
 		 */
-	        machdep->kvbase = symbol_value("_stext") & 0xffff0000UL;
+		machdep->kvbase = symbol_value("_stext") & ~KVBASE_MASK;
 		machdep->identity_map_base = machdep->kvbase;
 		machdep->is_kvaddr = arm_is_kvaddr;
 		machdep->is_uvaddr = arm_is_uvaddr;
@@ -1437,7 +1437,7 @@ arm_dis_filter(ulong vaddr, char *inbuf, unsigned int output_radix)
 		sprintf(buf1, "0x%lx <%s>\n", value,
 			value_to_symstr(value, buf2, output_radix));
 
-		sprintf(p1, buf1);
+		sprintf(p1, "%s", buf1);
 	}
 
 	console("    %s", inbuf);
