@@ -87,7 +87,7 @@ update_gdb_hooks(void)
         	exit_after_gdb_info : main_loop;
 	target_new_objfile_hook = NULL;
 #endif
-#if defined(GDB_7_0) || defined(GDB_7_3_1)
+#if defined(GDB_7_0) || defined(GDB_7_3_1) || defined(GDB_7_6)
 	deprecated_command_loop_hook = pc->flags & VERSION_QUERY ?
 		exit_after_gdb_info : main_loop;
 #endif
@@ -357,8 +357,8 @@ gdb_interface(struct gnu_request *req)
 		restart(0);
 
 	if (!req->fp) {
-		req->fp = pc->flags & RUNTIME ? fp : 
-			  CRASHDEBUG(1) ? fp : pc->nullfp;
+		req->fp = ((pc->flags & RUNTIME) || (pc->flags2 & ALLOW_FP)) ? 
+			fp : CRASHDEBUG(1) ? fp : pc->nullfp;
 	}
 
 	pc->cur_req = req;
@@ -894,6 +894,19 @@ gdb_line_number_callback(ulong pc, ulong low, ulong high)
 	return TRUE;
 }
 
+/*
+ *  Prevent gdb from trying to translate and print pointers
+ *  that are not kernel virtual addresses.  
+ */
+int
+gdb_print_callback(ulong addr)
+{
+	if (!addr)
+		return FALSE;
+	else
+		return IS_KVADDR(addr);
+}
+
 /*
  *  Used by gdb_interface() to catch gdb-related errors, if desired.
  */
@@ -918,7 +931,11 @@ gdb_error_hook(void)
 			gdb_command_string(pc->cur_gdb_cmd, buf1, TRUE), buf2);
 	}
 
+#ifdef GDB_7_6
+	do_cleanups(all_cleanups()); 
+#else
 	do_cleanups(NULL); 
+#endif
 
 	longjmp(pc->gdb_interface_env, 1);
 }
