@@ -35,7 +35,7 @@
 #include <asm/vmdump.h>                 /* for architecture-specific header */
 #endif
 
-#if defined(ARM) || defined(X86) || defined(PPC) || defined(S390) || defined(S390X)
+#if defined(ARM) || defined(X86) || defined(PPC) || defined(S390) || defined(S390X) || defined(ARM64)
 
 /*
  * Kernel header file for Linux crash dumps.
@@ -84,9 +84,11 @@ typedef struct _dump_header_asm_s {
 	/* the dump registers */
 #ifndef S390
 #ifndef S390X
+#ifndef ARM64
 	struct pt_regs       dha_regs;
 #endif
 #endif
+#endif
 
 } dump_header_asm_t;
 
