@@ -55,8 +55,105 @@ static int ppc64_get_cpu_map(void);
 static void ppc64_clear_machdep_cache(void);
 static void ppc64_vmemmap_init(void);
 static int ppc64_get_kvaddr_ranges(struct vaddr_range *);
+static uint get_ptetype(ulong pte);
+static int is_hugepage(ulong pte);
+static int is_hugepd(ulong pte);
+static ulong hugepage_dir(ulong pte);
+
+static inline uint get_ptetype(ulong pte)
+{
+	uint pte_type = 0; /* 0: regular entry; 1: huge pte; 2: huge pd */
+
+	if (is_hugepage(pte))
+		pte_type = 1;
+	else if (is_hugepd(pte))
+		pte_type = 2;
+
+	return pte_type;
+}
+
+static int is_hugepage(ulong pte)
+{
+	/*
+	 * leaf pte for huge page, bottom two bits != 00
+	 */
+	return ((pte & HUGE_PTE_MASK) != 0x0);
+}
+
+static inline int is_hugepd(ulong pte)
+{
+	if (THIS_KERNEL_VERSION >= LINUX(3,10,0)) {
+		/*
+		 * hugepd pointer, bottom two bits == 00 and next 4 bits
+		 * indicate size of table
+		*/
+		return (((pte & HUGE_PTE_MASK) == 0x0) &&
+			((pte & HUGEPD_SHIFT_MASK) != 0));
+	} else
+		return ((pte & PD_HUGE) == 0x0);
+}
+
+static inline ulong hugepage_dir(ulong pte)
+{
+	if (THIS_KERNEL_VERSION >= LINUX(3,10,0))
+		return (ulong)(pte & ~HUGEPD_SHIFT_MASK);
+	else
+		return (ulong)((pte & ~HUGEPD_SHIFT_MASK) | PD_HUGE);
+}
+
+static int book3e_is_kvaddr(ulong addr)
+{
+	return (addr >= BOOK3E_VMBASE);
+}
+
+
+static int book3e_is_vmaddr(ulong addr)
+{
+	return (addr >= BOOK3E_VMBASE) && (addr < machdep->identity_map_base);
+}
+
+static int ppc64_is_vmaddr(ulong addr)
+{
+	return (vt->vmalloc_start && addr >= vt->vmalloc_start);
+}
+
+
+struct machine_specific ppc64_machine_specific = { 
+	.hwintrstack = { 0 }, 
+	.hwstackbuf = 0,
+	.hwstacksize = 0,
+	.pte_shift = PTE_SHIFT,
+	._page_present = 0x1UL,
+	._page_user = 0x2UL,
+	._page_rw = 0x4UL,
+	._page_guarded = 0x8UL,
+	._page_coherent = 0x10UL,
+	._page_no_cache = 0x20UL,
+	._page_writethru = 0x40UL,
+	._page_dirty = 0x80UL,
+	._page_accessed = 0x100UL,
+	.is_kvaddr = generic_is_kvaddr,
+	.is_vmaddr = ppc64_is_vmaddr,
+};
+
+struct machine_specific book3e_machine_specific = {
+	.hwintrstack = { 0 },
+	.hwstackbuf = 0,
+	.hwstacksize = 0,
+	.pte_shift = PTE_SHIFT_L4_BOOK3E_64K, 
+	._page_present = 0x1UL,
+	._page_user = 0xCUL,
+	._page_rw = 0x30UL,
+	._page_guarded = 0x100000UL,
+	._page_coherent = 0x200000UL,
+	._page_no_cache = 0x400000UL,
+	._page_writethru = 0x800000UL,
+	._page_dirty = 0x1000UL,
+	._page_accessed = 0x40000UL,
+	.is_kvaddr = book3e_is_kvaddr,
+	.is_vmaddr = book3e_is_vmaddr,
+};
 
-struct machine_specific ppc64_machine_specific = { { 0 }, 0, 0 };
 
 /*
  *  Do all necessary machine-specific setup here.  This is called several
@@ -103,6 +200,12 @@ ppc64_init(int when)
                 * Also moved the following code block from
                 * PRE_SYMTAB case here.
                 */
+		if (symbol_exists("interrupt_base_book3e")) {
+			machdep->machspec = &book3e_machine_specific;
+			machdep->flags |= BOOK3E;
+			machdep->kvbase = BOOK3E_VMBASE;
+		} else
+			machdep->kvbase = symbol_value("_stext");
                 if (symbol_exists("__hash_page_64K"))
                         machdep->pagesize = PPC64_64K_PAGE_SIZE;
                 else
@@ -119,9 +222,8 @@ ppc64_init(int when)
 		if ((machdep->machspec->level4 = (char *)malloc(PAGESIZE())) == NULL)
 			error(FATAL, "cannot malloc level4 space.");
 
-	        machdep->kvbase = symbol_value("_stext");
-		machdep->identity_map_base = machdep->kvbase;
-                machdep->is_kvaddr = generic_is_kvaddr;
+		machdep->identity_map_base = symbol_value("_stext"); 
+                machdep->is_kvaddr = machdep->machspec->is_kvaddr; 
                 machdep->is_uvaddr = generic_is_uvaddr;
 	        machdep->eframe_search = ppc64_eframe_search;
 	        machdep->back_trace = ppc64_back_trace_cmd;
@@ -169,12 +271,21 @@ ppc64_init(int when)
 			struct machine_specific *m = machdep->machspec;
 			if (machdep->pagesize == 65536) {
 				/* 64K pagesize */
-				m->l1_index_size = PTE_INDEX_SIZE_L4_64K;
-				m->l2_index_size = PMD_INDEX_SIZE_L4_64K;
-				m->l3_index_size = PUD_INDEX_SIZE_L4_64K;
-				m->l4_index_size = PGD_INDEX_SIZE_L4_64K;
-				m->pte_shift = symbol_exists("demote_segment_4k") ?
-					PTE_SHIFT_L4_64K_V2 : PTE_SHIFT_L4_64K_V1; 
+				if (THIS_KERNEL_VERSION >= LINUX(3,10,0)) {
+					m->l1_index_size = PTE_INDEX_SIZE_L4_64K_3_10;
+					m->l2_index_size = PMD_INDEX_SIZE_L4_64K_3_10;
+					m->l3_index_size = PUD_INDEX_SIZE_L4_64K;
+					m->l4_index_size = PGD_INDEX_SIZE_L4_64K_3_10;
+
+				} else {
+					m->l1_index_size = PTE_INDEX_SIZE_L4_64K;
+					m->l2_index_size = PMD_INDEX_SIZE_L4_64K;
+					m->l3_index_size = PUD_INDEX_SIZE_L4_64K;
+					m->l4_index_size = PGD_INDEX_SIZE_L4_64K;
+				}
+				if (!(machdep->flags & BOOK3E))
+					m->pte_shift = symbol_exists("demote_segment_4k") ?
+						PTE_SHIFT_L4_64K_V2 : PTE_SHIFT_L4_64K_V1; 
 				m->l2_masked_bits = PMD_MASKED_BITS_64K;
 			} else {
 				/* 4K pagesize */
@@ -182,7 +293,8 @@ ppc64_init(int when)
 				m->l2_index_size = PMD_INDEX_SIZE_L4_4K;
 				m->l3_index_size = PUD_INDEX_SIZE_L4_4K;
 				m->l4_index_size = PGD_INDEX_SIZE_L4_4K;
-				m->pte_shift = PTE_SHIFT_L4_4K; 
+				m->pte_shift = (machdep->flags & BOOK3E) ? 
+					PTE_SHIFT_L4_BOOK3E_4K : PTE_SHIFT_L4_4K; 
 				m->l2_masked_bits = PMD_MASKED_BITS_4K;
 			}
 
@@ -354,7 +466,7 @@ ppc64_dump_machdep_table(ulong arg)
         fprintf(fp, ")\n");
 
 	fprintf(fp, "             kvbase: %lx\n", machdep->kvbase);
-	fprintf(fp, "  identity_map_base: %lx\n", machdep->kvbase);
+	fprintf(fp, "  identity_map_base: %lx\n", machdep->identity_map_base);
         fprintf(fp, "           pagesize: %d\n", machdep->pagesize);
         fprintf(fp, "          pageshift: %d\n", machdep->pageshift);
         fprintf(fp, "           pagemask: %llx\n", machdep->pagemask);
@@ -384,7 +496,9 @@ ppc64_dump_machdep_table(ulong arg)
 	fprintf(fp, "         dis_filter: ppc64_dis_filter()\n");
 	fprintf(fp, "           cmd_mach: ppc64_cmd_mach()\n");
 	fprintf(fp, "       get_smp_cpus: ppc64_get_smp_cpus()\n");
-        fprintf(fp, "          is_kvaddr: generic_is_kvaddr()\n");
+        fprintf(fp, "          is_kvaddr: %s\n", 
+		machdep->is_kvaddr == book3e_is_kvaddr ? 
+		"book3e_is_kvaddr()" : "generic_is_kvaddr()"); 
         fprintf(fp, "          is_uvaddr: generic_is_uvaddr()\n");
         fprintf(fp, "       verify_paddr: generic_verify_paddr()\n");
         fprintf(fp, "  get_kvaddr_ranges: ppc64_get_kvaddr_ranges()\n");
@@ -410,7 +524,13 @@ ppc64_dump_machdep_table(ulong arg)
                         machdep->cmdline_args[i] : "(unused)");
         }
 	fprintf(fp, "           machspec: %lx\n", (ulong)machdep->machspec);
-	fprintf(fp, "     hwintrstack[%d]: ", NR_CPUS);
+	fprintf(fp, "            is_kvaddr: %s\n", 
+		machdep->machspec->is_kvaddr == book3e_is_kvaddr ? 
+		"book3e_is_kvaddr()" : "generic_is_kvaddr()");
+	fprintf(fp, "            is_vmaddr: %s\n", 
+		machdep->machspec->is_vmaddr == book3e_is_vmaddr ? 
+		"book3e_is_vmaddr()" : "ppc64_is_vmaddr()");
+	fprintf(fp, "    hwintrstack[%d]: ", NR_CPUS);
        	for (c = 0; c < NR_CPUS; c++) {
 		for (others = 0, i = c; i < NR_CPUS; i++) {
 			if (machdep->machspec->hwintrstack[i])
@@ -561,6 +681,7 @@ ppc64_vtop_level4(ulong vaddr, ulong *level4, physaddr_t *paddr, int verbose)
 	ulong *page_table;
 	ulong level4_pte, pgd_pte, pmd_pte;
 	ulong pte;
+	uint  hugepage_type = 0; /* 0: regular entry; 1: huge pte; 2: huge pd */
 
 	if (verbose)
 		fprintf(fp, "PAGE DIRECTORY: %lx\n", (ulong)level4);
@@ -573,6 +694,12 @@ ppc64_vtop_level4(ulong vaddr, ulong *level4, physaddr_t *paddr, int verbose)
 	if (!level4_pte)
 		return FALSE;
 
+	hugepage_type = get_ptetype(level4_pte);
+	if (hugepage_type) {
+		pte = level4_pte;
+		goto out;
+	}
+
 	/* Sometimes we don't have level3 pagetable entries */
 	if (machdep->machspec->l3_index_size != 0) {
 		page_dir = (ulong *)((ulong *)level4_pte + PGD_OFFSET_L4(vaddr));
@@ -583,6 +710,12 @@ ppc64_vtop_level4(ulong vaddr, ulong *level4, physaddr_t *paddr, int verbose)
 			fprintf(fp, "  PGD: %lx => %lx\n", (ulong)page_dir, pgd_pte);
 		if (!pgd_pte)
 			return FALSE;
+
+		hugepage_type = get_ptetype(pgd_pte);
+		if (hugepage_type) {
+			pte = pgd_pte;
+			goto out;
+		}
 	} else {
 		pgd_pte = level4_pte;
 	}
@@ -597,6 +730,12 @@ ppc64_vtop_level4(ulong vaddr, ulong *level4, physaddr_t *paddr, int verbose)
 	if (!(pmd_pte))
 		return FALSE;
 
+	hugepage_type = get_ptetype(pmd_pte);
+	if (hugepage_type) {
+		pte = pmd_pte;
+		goto out;
+	}
+
 	page_table = (ulong *)(pmd_pte & ~(machdep->machspec->l2_masked_bits))
 			 + (BTOP(vaddr) & (machdep->machspec->ptrs_per_l1 - 1));
 	if (verbose)
@@ -620,17 +759,37 @@ ppc64_vtop_level4(ulong vaddr, ulong *level4, physaddr_t *paddr, int verbose)
 	if (!pte)
 		return FALSE;
 
-	*paddr = PAGEBASE(PTOB(pte >> machdep->machspec->pte_shift)) 
-			+ PAGEOFFSET(vaddr);
+out:
+	if (hugepage_type) {
+		if (hugepage_type == 2) {
+			/* TODO: Calculate the offset within the huge page
+			 * directory for this huge page to get corresponding
+			 * physical address. In the current form, it may
+			 * return the physical address of the first huge page
+			 * in this directory for all the huge pages
+			 * in this huge page directory.
+			 */
+			readmem(hugepage_dir(pte), KVADDR, &pte, sizeof(pte),
+				"hugepd_entry", RETURN_ON_ERROR);
+		}
+		/* TODO: get page offset for huge pages based on page size */
+		*paddr = PAGEBASE(PTOB(pte >> machdep->machspec->pte_shift));
+	} else {
+		*paddr = PAGEBASE(PTOB(pte >> machdep->machspec->pte_shift))
+				+ PAGEOFFSET(vaddr);
+	}
 
 	if (verbose) {
-		fprintf(fp, " PAGE: %lx\n\n", PAGEBASE(*paddr));
+		if (hugepage_type)
+			fprintf(fp, " HUGE PAGE: %lx\n\n", PAGEBASE(*paddr));
+		else
+			fprintf(fp, " PAGE: %lx\n\n", PAGEBASE(*paddr));
 		ppc64_translate_pte(pte, 0, machdep->machspec->pte_shift);
 	}
 
 	return TRUE;
 }
-	
+
 /*
  *  Translates a user virtual address to its physical address.  cmd_vtop()
  *  sets the verbose flag so that the pte translation gets displayed; all
@@ -899,6 +1058,12 @@ ppc64_processor_speed(void)
         if (machdep->mhz)
                 return(machdep->mhz);
 
+	if (symbol_exists("ppc_proc_freq")) {
+		get_symbol_data("ppc_proc_freq", sizeof(ulong), &mhz);
+		mhz /= 1000000;
+		return (machdep->mhz = mhz);
+	}
+
         if(symbol_exists("allnodes")) {
                 get_symbol_data("allnodes", sizeof(void *), &node);
                 while(node) {
@@ -2313,7 +2478,7 @@ ppc64_dis_filter(ulong vaddr, char *inbuf, unsigned int output_radix)
 		sprintf(buf1, "0x%lx <%s>\n", value,	
 			value_to_symstr(value, buf2, output_radix));
 
-		sprintf(p1, buf1);
+		sprintf(p1, "%s", buf1);
 	}
 
 	console("    %s", inbuf);
