@@ -1,8 +1,8 @@
 /* remote.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002, 2003, 2004, 2005 David Anderson
- * Copyright (C) 2002, 2003, 2004, 2005 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002, 2003, 2004, 2005, 2009, 2011 David Anderson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2009, 2011 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1116,8 +1116,10 @@ daemon_proc_version(char *buf)
                 return FALSE;
 
         if (fread(buf, sizeof(char),
-                BUFSIZE-1, pipe) <= 0)
+                BUFSIZE-1, pipe) <= 0) {
+		pclose(pipe);
                 return FALSE;
+	}
 
         pclose(pipe);
 
@@ -2953,8 +2955,8 @@ copy_remote_file(struct remote_file *rfp, int fd, char *file, char *ttystr)
 	long pct, last;
         ulong size, offset, filesize;
         ulong ret, req, tot;
-	int sysret;
-	ssize_t bytes;
+	int sysret ATTRIBUTE_UNUSED;
+	ssize_t bytes ATTRIBUTE_UNUSED;
 
 	last = -1;
 	lseek(fd, 0, SEEK_SET);
