@@ -1,8 +1,8 @@
 /* s390.c - core analysis suite
  *
  * Copyright (C) 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2009, 2010, 2012 David Anderson
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2009, 2010, 2012 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002-2006, 2009-2010, 2012-2013 David Anderson
+ * Copyright (C) 2002-2006, 2009-2010, 2012-2013 Red Hat, Inc. All rights reserved.
  * Copyright (C) 2005, 2006, 2010 Michael Holzheu, IBM Corporation
  *
  * This program is free software; you can redistribute it and/or modify
@@ -1012,7 +1012,7 @@ s390_dis_filter(ulong vaddr, char *inbuf, unsigned int output_radix)
 		sprintf(buf1, "0x%lx <%s>\n", value,
 			value_to_symstr(value, buf2, output_radix));
 
-		sprintf(p1, buf1);
+		sprintf(p1, "%s", buf1);
 	}
 
 	console("    %s", inbuf);
