@@ -1,15 +1,18 @@
 /* 
  * xendump.h
  *
- * Copyright (C) 2006, 2007 David Anderson
- * Copyright (C) 2006, 2007 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2006, 2007, 2009, 2010 David Anderson
+ * Copyright (C) 2006, 2007, 2009, 2010 Red Hat, Inc. All rights reserved.
  *
- * This software may be freely redistributed under the terms of the
- * GNU General Public License.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  */
 #include <endian.h>
 #include <elf.h>
@@ -30,6 +33,20 @@ typedef struct xc_core_header {
     unsigned int xch_pages_offset;
 } xc_core_header_t;
 
+/*
+ *  Based upon the original xensource xc_core_header struct above, 
+ *  but with unsigned long offset values so that it can be used
+ *  with the original dumpfile format and new ELF-style format.
+ */
+struct xen_core_header {
+    unsigned int xch_magic;
+    unsigned int xch_nr_vcpus;
+    unsigned int xch_nr_pages;
+    unsigned long xch_ctxt_offset;
+    unsigned long xch_index_offset;
+    unsigned long xch_pages_offset;
+};
+
 struct pfn_offset_cache {
 	off_t file_offset;
 	ulong pfn;
@@ -67,7 +84,7 @@ struct xendump_data {
 	struct xc_core_data {
 		int p2m_frames;
 		ulong *p2m_frame_index_list;
-		struct xc_core_header header;
+		struct xen_core_header header;
 		int elf_class;
 		uint64_t format_version;
 		off_t elf_strtab_offset;
